#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>


/* USART baud rate calculation */
#define CLK_PER							(F_CPU)
#define USART_BAUD_RATE(F_BAUD)			((float)CLK_PER * 64. / (16 * (float)F_BAUD) + 0.5)


static int usart_tx(char c, FILE *stream);
static void usart_rx(FILE *stream);


/* When you use ANCI-C key input API... */
static FILE avr_stdio = FDEV_SETUP_STREAM(usart_tx, usart_rx, _FDEV_SETUP_RW);




static int usart_tx(char c, FILE *stream)
{
	if(c == '\n')
	{
		while ((USART0.STATUS & USART_DREIF_bm) == 0);
		USART0.TXDATAL = 0x0D;	// carriage return
	}

	while ((USART0.STATUS & USART_DREIF_bm) == 0);
	USART0.TXDATAL = c;

	while ((USART0.STATUS & USART_TXCIF_bm) == 1);


	return 0;
}




static void usart_rx(FILE *stream)
{
	volatile char data;

	while ((USART0.STATUS & USART_RXCIF_bm) == 0);

	data = USART0.RXDATAL;

	// Just for test...
	//usart_tx(data, stream);
}



static void usart_rx_intr_enable(void)
{
	USART0.CTRLA |= USART_RXCIE_bm;
}



ISR(USART0_RXC_vect)
{
	unsigned char rcv_data;

	while ((USART0.STATUS & USART_RXCIF_bm) == 0);

	rcv_data = USART0.RXDATAL;

	// Just for test...
	//uart1_tx(rcv_data, NULL);
}




void USART_Printf_Init(uint32_t baudrate)
{
	int8_t sigrow_value = SIGROW.OSC20ERR5V;	// Read signed error

	PORTB.DIRSET |= (1 << PIN2_bp);		//PB2 as output
	PORTB.OUTCLR |= (1 << PIN2_bp);		// initial level false
	
	// USART baud rate initialization as mentioned in the datasheet
	baudrate *= (1024 + sigrow_value);			// Sum resolution + Error
	baudrate /=  1024;							// Divide by resolution

	USART0.BAUD = (uint16_t)USART_BAUD_RATE(baudrate);	// Set adjusted baud rate

	USART0.CTRLB = (1 << USART_RXEN_bp) | (1 << USART_TXEN_bp) | (1 << USART_SFDEN_bp);	// Transmitter enable (normal mode)

	// Assign device(avr_stdio) to stdout & stdin.
	stdout = stdin = &avr_stdio;

	_delay_ms(10);
}
